/*Andrew Willouer
COMS 4115 Programming Languages and Translators
8/10/2009*/

(*Parser.mly
Parser takes tokens from scanner and parses them into expressions, statements, variables and so forth for the interpreter*)

%{ open Ast %}

%token SEMI LPAREN RPAREN LBRACE RBRACE COMMA
%token LBRACKET RBRACKET PERIOD MATCH
%token PLUS MINUS TIMES DIVIDE ASSIGN
%token EQ NEQ LT LEQ GT GEQ
%token AND OR
%token RETURN IF ELSE FOR WHILE INT
%token HANDS CARDS VALUE SETVALUE DEAL DECKS SHUFFLE
%token PLAYERS
%token DEALER CARDNAME NUMBERDECKS
%token VAR GLOBALVAR
%token GLOBALS RULES
%token PRINT READ
%token BOOL INT STRING
%token <int> INTLITERAL
%token <string> STRINGLITERAL
%token <string> ID
%token <bool> TRUE FALSE 
%token <string> CARDLITERAL  

%token EOF
 
%nonassoc NOELSE
%nonassoc ELSE

%left ASSIGN
%left EQ NEQ 
%left AND OR
%left LT GT LEQ GEQ
%left PLUS MINUS 
%left TIMES DIVIDE
%right READ PRINT PRINTHAND
%left GLOBALVAR VAR PLAYER DECKS
%left BOOL INT STRING CARDS HANDS
%right DEALER CARDNAME NUMBERDECKS
%left VALUE SETVALUE 

%start program
%type <Ast.program> program 

%% 
(*Program contains variable declartions, function declarations*)
program:
  /* nothing */ { [], [] }
 | program vdecl { ($2 :: fst $1), snd $1 }
 | program fdecl { fst $1, ($2 :: snd $1) }

(*Function list is a list of function declarations*)  
funcs_list: 
  /* nothing */ { [] }
 | funcs_list fdecl { $2 :: $1 }

(*Function declaration*)
fdecl:
   ID LPAREN formals_opt RPAREN LBRACE vdecl_list stmt_list RBRACE
     { { fname = $1;
	 formals = $3;
	 locals = List.rev $6;
	 body = List.rev $7 } }

formals_opt:
    /* nothing */ { [] }
  | formal_list   { List.rev $1 }

formal_list:
    vdecl                   { [$1] }
  | formal_list COMMA vdecl { $3 :: $1 }

(*Variable declaration list is a list of variables*)  
vdecl_list:
    /* nothing */    { [] }
  | vdecl_list vdecl SEMI { $2 :: $1 }

(*Variable declaratoin consists of keyword VAR followed by ID 
    or DECKS followed by ID or PLAYERS followed by ID*)
vdecl:
   VAR ID { $2 }
   | VAR GLOBALVAR ID { $3 }
   | DECKS ID { $2 }
   | PLAYERS ID { $2 }

(*List of statements*)  
stmt_list:
   /* nothing */  { [] }
   | stmt_list stmt { $2 :: $1 } 

(*Statements consist of Print, Read, Return, If, For, While, Deal, Value*)  
stmt:
    expr SEMI { Expr($1) }
  | PRINT LPAREN expr RPAREN SEMI { Print($3) }
  | READ LPAREN var RPAREN SEMI { Read($3) }
  | RETURN expr_opt SEMI { Return($2) }
  | IF LPAREN expr RPAREN LBRACE stmt_list RBRACE %prec NOELSE { If($3, $6, []) }
  | IF LPAREN expr RPAREN LBRACE stmt_list RBRACE ELSE LBRACE stmt_list RBRACE    { If($3, $6, $10) }
  | FOR LPAREN expr_opt SEMI expr_opt SEMI expr_opt RPAREN LBRACE stmt RBRACE
    { For($3, $5, $7, $10) }
  | WHILE LPAREN expr RPAREN LBRACE stmt_list RBRACE { While($3, $6) }
  | DEAL LPAREN var RPAREN SEMI {Deal ($3) }
  | VALUE LPAREN expr RPAREN SEMI {Value ($3) }

expr_opt:
    /* nothing */ { Noexpr }
  | expr          { $1 }

  
(*Expressions consist of the operators*)  
expr:
  | TRUE             { Boolliteral(true)}
  | FALSE            { Boolliteral(false)}
  | CARDLITERAL      { Cardliteral($1) }
  | INTLITERAL       { Intliteral($1) }
  | STRINGLITERAL    { Stringliteral($1) } 
  | var              { Variable($1) }
  | expr PLUS   expr { Binop($1, Add,   $3) }
  | expr MINUS  expr { Binop($1, Sub,   $3) }
  | expr TIMES  expr { Binop($1, Mult,  $3) }
  | expr DIVIDE expr { Binop($1, Div,   $3) }
  | expr EQ     expr { Binop($1, Equal, $3) }
  | expr NEQ    expr { Binop($1, Neq,   $3) }
  | expr LT     expr { Binop($1, Less,  $3) }
  | expr LEQ    expr { Binop($1, Leq,   $3) }
  | expr GT     expr { Binop($1, Greater,  $3) }
  | expr GEQ    expr { Binop($1, Geq,   $3) } 
  | expr AND    expr { Binop($1, And, $3) }
  | expr OR     expr { Binop($1, Or, $3) }
  | var ASSIGN expr  { Assign($1, $3) }
  | ID LPAREN actuals_opt RPAREN { Call($1, $3) }
  | LPAREN expr RPAREN { $2 }

var:
     ID  { VarExp($1, Local) }
   | GLOBALVAR ID { VarExp($2, Global) }
  
actuals_opt:
    /* nothing */ { [] }
  | actuals_list  { List.rev $1 }

actuals_list:
    expr                    { [$1] }
  | actuals_list COMMA expr { $3 :: $1 }
