(*Andrew Willouer
COMS 4115 Programming Languages and Translators
8/10/2009*)

(*Scanner.mll
Scanner generates tokens for the parser*)

{ open Parser }

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "/*"     { comment lexbuf }           (* Comments *)
| '('      { LPAREN }                    (*Punctuation*)
| ')'      { RPAREN }
| '{'      { LBRACE }
| '}'      { RBRACE }
| '['      { LBRACKET }
| ']'      { RBRACKET }
| '.'      { PERIOD}
| ';'      { SEMI }
| ','      { COMMA }
| '?'      { MATCH }
| '+'      { PLUS }                        (*Operators*)
| '-'      { MINUS } 
| '*'      { TIMES }
| '/'      { DIVIDE }
| '='      { ASSIGN }
| "=="     { EQ }
| "!="     { NEQ }
| "&&"     { AND } 
| "||"     { OR } 
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| "if"     { IF }                       (*Keywords*)
| "else"   { ELSE }
| "for"    { FOR }
| "while"  { WHILE }
| "return" { RETURN }
| "int"    { INT }
| "bool"   { BOOL }
| "true"   { TRUE(true) }
| "false"  { FALSE(false) }
| "Print"  { PRINT }
| "Hands"  { HANDS }
| "Cards"  { CARDS }
| "Deal"   { DEAL }
| "Decks"  { DECKS }
| "Value"  { VALUE }
| "Read"  { READ }  
| "var"    { VAR }
| "Player" { PLAYERS }
| "@"      { GLOBALVAR }
| "Globals" { GLOBALS }
| "2S"     { CARDLITERAL("2S") }     (*Card Types*)
| "2C"     { CARDLITERAL("2C") }
| "2D"     { CARDLITERAL("2D") }
| "2H"     { CARDLITERAL("2H") }
| "3S"     { CARDLITERAL("3S") }
| "3C"     { CARDLITERAL("3C") }
| "3D"     { CARDLITERAL("3D") }
| "3H"     { CARDLITERAL("3H") }
| "4S"     { CARDLITERAL("4S") }
| "4C"     { CARDLITERAL("4C") }
| "4D"     { CARDLITERAL("4D") }
| "4H"     { CARDLITERAL("4H") }
| "5S"     { CARDLITERAL("5S") }
| "5C"     { CARDLITERAL("5C") }
| "5D"     { CARDLITERAL("5D") }
| "5H"     { CARDLITERAL("5H") }
| "6S"     { CARDLITERAL("6S") }
| "6C"     { CARDLITERAL("6C") }
| "6D"     { CARDLITERAL("6D") }
| "6H"     { CARDLITERAL("6H") }
| "7S"     { CARDLITERAL("7S") }
| "7C"     { CARDLITERAL("7C") }
| "7D"     { CARDLITERAL("7D") } 
| "7H"     { CARDLITERAL("7H") }
| "8S"     { CARDLITERAL("8S") }
| "8C"     { CARDLITERAL("8C") }
| "8D"     { CARDLITERAL("8D") }
| "8H"     { CARDLITERAL("8H") }
| "9S"     { CARDLITERAL("9S") }
| "9C"     { CARDLITERAL("9C") }
| "9D"     { CARDLITERAL("9D") }
| "9H"     { CARDLITERAL("9H") }
| "10S"     { CARDLITERAL("10S") }
| "10C"     { CARDLITERAL("10C") }
| "10D"     { CARDLITERAL("10D") }
| "10H"     { CARDLITERAL("10H") }
| "JS"     { CARDLITERAL("JS") }
| "JC"     { CARDLITERAL("JC") }
| "JD"     { CARDLITERAL("JD") }
| "JH"     { CARDLITERAL("JH") }
| "QS"     { CARDLITERAL("QS") }
| "QC"     { CARDLITERAL("QC") }
| "QD"     { CARDLITERAL("QD") }
| "QH"     { CARDLITERAL("QH") }
| "KS"     { CARDLITERAL("KS") }
| "KC"     { CARDLITERAL("KC") }
| "KD"     { CARDLITERAL("KD") }
| "KH"     { CARDLITERAL("KH") } 
| "AS"     { CARDLITERAL("AS") } 
| "AC"     { CARDLITERAL("AC") }
| "AD"     { CARDLITERAL("AD") }
| "AH"     { CARDLITERAL("AH") }
| (['2'-'9'] | "10" | "J" | "Q" | "K" | "A")['S' 'C' 'D' 'H'] as lxm { CARDLITERAL(lxm) }
| ['0'-'9']+ as lxm { INTLITERAL(int_of_string lxm) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| '\"'[^ '\"']*'\"' as lxm { STRINGLITERAL(String.sub lxm 1 ((String.length lxm) -2)) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "*/" { token lexbuf }
| _    { comment lexbuf }
